/************************************************************************************/
/*    Manufacture: OLIMEX                                           	            */
/*	  COPYRIGHT (C) 2011							    */
/*    Designed by:  Penko Todorov Bozhkov	        		            */
/*    Module Name:  USART		            		                    */
/*    File   Name:  usart.c					                    */
/*    Revision Identifier:	Rev.A						    */
/*	  Revision History:							    */
/*		-> Initial:	Writed RAM related functions.                       */
/*		-> Rev.A:	Added ROM related functions.(10.03.2011)            */
/************************************************************************************/
#include  <msp430.h>
#include "usart.h"

//Global Variables initialization:
volatile unsigned char USART_Flags_Register = 0;
unsigned char USART_Received_Data = 0;

void usart_wait(volatile unsigned int cycles){
  while(cycles){cycles--;}
}


/****************************************************************************/
/*  Function name: USART_Init                                               */
/*  	Parameters                                                          */
/*          Input   :  No                                                   */
/*          Output  :  No	                                            */
/*	Action: Initialize USART operation mode and speed.		    */
/****************************************************************************/
// desired baud rate: 9600
void USART_Init(void)
{ 
  /*
  UCA0CTL1 |= UCSWRST;                      // **Put state machine in reset**
  UCA0CTL1 |= UCSSEL_2;                     // SMCLK
  UCA0BR0 = 6;                              // 1MHz 9600 (see User's Guide)
  UCA0BR1 = 0;                              // 1MHz 9600
  UCA0MCTL = UCBRS_0 + UCBRF_13 + UCOS16;   // Modln UCBRSx=0, UCBRFx=0, over sampling
 
  P1DIR |= BIT6;                            // Set P1.6 as TX output
  P1SEL |= BIT5 + BIT6;                     // Select P1.5 & P1.6 to UART function
  
  UCA0CTL1 &= (~UCSWRST);                   // **Initialize USCI state machine**
  UCA0IE |= UCRXIE;                         // Enable USCI_A0 RX interrupt
  UCA0IFG &= (~UCRXIFG);                    // Clear RX interrupt falg
  */
  // 2.1. USCI_A0
  // ============
  // The recommended USCI initialization/re-configuration process is:
  UCA0CTL1 |= 0x01;  // 1) Set UCSWRST
  // 2) Initialize all USCI registers with UCSWRST = 1 (including UCAxCTL1)
  UCA0CTL1 |= UCSSEL_2; // SMCLK is selected ~ 1.1MHz by default
  UCA0BR0 = 104;        // 1MHz 9600 
  UCA0BR1 = 0;          // 1MHz 9600
  UCA0MCTL = UCBRS0;    // Modulation UCBRSx = 1
  // 3) Configure ports.
  P3SEL |= 0x30;         // P3.4,5 = USCI_A0 TXD/RXD
  // 4) Clear UCSWRST via software
  UCA0CTL1 &= ~UCSWRST; // **Initialize USCI state machine**
  // 5) Enable interrupts (optional)
  IE2 |= UCA0RXIE;     // Enable USCI_A0 RX interrupt
  //IFG2 &= ~0x01;       // Clear UCA0 RXIFG
  
}


/****************************************************************************/
/*  Function name: USART_Send_Data                                          */
/*  	Parameters                                                          */
/*          Input   :  Data                                                 */
/*          Output  :  No                                                   */
/*	Action: Send one data byte to HyperTerminal via USART.              */
/****************************************************************************/
void USART_Send_Data(unsigned char Data){
  
  while(USART_BUSY);	// Check USCI busy flag
  USART_TxData_Reg = Data;
}


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//##### Functions related with strings and so on placed in ROM(Flash) memory #####
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#if defined(ROM_Functions_Enabled)

#define Menu_TableNumberRows	7

// 1.Put the strings in Program Space
const char row_0[] = ("\n\r\n\r****************** Select Test: *********************");
const char row_1[] = ("\f\r******************************************************");
const char row_2[] = ("\n\r* Welcom to OLIMEX MSP430-G2744BP_UART demo program! *");
const char row_3[] = ("\n\r******************************************************");
const char row_4[] = ("\n\r*    Press any key to receive its echo(ASCII+1)!     *");
const char row_5[] = ("\n\r******************************************************");
const char row_6[] = ("\n\r\n\r>\0");

// 2.Put "Menue_Table" in Program Space, where "Menue_Table" is an array of pointers to  
//   characters (strings), where each pointer is a pointer to the Program Space.
const char * Menu_Table[] = 
{
  row_0,
  row_1,
  row_2,
  row_3,
  row_4,
  row_5,
  row_6
};


/**********************************************************************************/
/*  Function name: USART_Send_ROM_String                                          */
/*  	Parameters                                                                */
/*          Input   :  String_pointer                                             */
/*          Output  :  No	                                                  */
/*	Action: Send string stored in Program Space to HyperTerminal via USART.	  */
/**********************************************************************************/
void USART_Send_ROM_String(const char *String_pointer){
  while((*String_pointer)){
    USART_Send_Data((*String_pointer));
    String_pointer++;	
  }	
}


/**********************************************************************************/
/*  Function name: USART_Send_ROM_Menu_Begin                                      */
/*  	Parameters                                                                */
/*          Input   :  No			                                  */
/*          Output  :  No	                                                  */
/*	Action: Send Menu stored in Program Space to HyperTerminal via USART.	  */
/**********************************************************************************/
void USART_Send_ROM_Menu_Begin(void){
  volatile unsigned char i=0;

  // String placed in ROM
  for(i=0;i<Menu_TableNumberRows;i++){
    if(i==0){	continue;	}
      USART_Send_ROM_String( (Menu_Table[i]) );
  }
}


/**********************************************************************************/
/*  Function name: USART_Send_ROM_Menu_Restore                                    */
/*  	Parameters                                                                */
/*          Input   :  No			                             	  */
/*          Output  :  No	                                                  */
/*	Action: Send Menu stored in Program Space to HyperTerminal via USART.	  */
/**********************************************************************************/
void USART_Send_ROM_Menu_Restore(void){
  unsigned char i=0;
	
  // String placed in ROM
  for(i=0;i<Menu_TableNumberRows;i++){
    if((i==1)|(i==2)|(i==3)){ continue;	}
      USART_Send_ROM_String(Menu_Table[i]);
  }
}

#endif	//ROM_Functions_Enabled


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//#####    Functions related with strings and so on placed in RAM memory     #####
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#if defined(RAM_Functions_Enabled)

/**********************************************************************************/
/*  Function name: USART_Send_RAM_String                                          */
/*  	Parameters                                                                */
/*          Input   :  *String_pointer                                            */
/*          Output  :  No	                                                  */
/*	Action: Send string stored in RAM to HyperTerminal via USART.  		  */
/**********************************************************************************/
void USART_Send_RAM_String(char *String_pointer){
  while(*String_pointer){
    USART_Send_Data(*String_pointer);
    String_pointer++;	
  }	
}


/**********************************************************************************/
/*  Function name: USART_Send_RAM_Menu_Begin                                      */
/*  	Parameters                                                                */
/*          Input   :  No			                                  */
/*          Output  :  No	                                                  */
/*	Action: First TEST Menu sending to HyperTerminal.        		  */
/**********************************************************************************/
void USART_Send_RAM_Menu_Begin(void){
  // Write TEST Menu to HyperTerminal:
  USART_Send_RAM_String("\f\r*****************************************************"\
                        "\n\r*  Welcom to OLIMEX MSP430-G2744_UART demo program! *"\
                        "\n\r*****************************************************"\
                        "\n\r*   Press any key to receive its echo(ASCII+1)!     *"\
                        "\n\r*****************************************************"\
                        "\n\r\n\r>\0"); 
}


/**********************************************************************************/
/*  Function name: USART_Send_RAM_Menu_Restore                                    */
/*  	Parameters                                                                */
/*          Input   :  No				                          */
/*          Output  :  No	                                                  */
/*	Action: Restore TEST Menu when selected test is completed. 		  */
/**********************************************************************************/
void USART_Send_RAM_Menu_Restore(void){
  // Restor this Menu to HyperTerminal
  USART_Send_RAM_String("\n\r\n\r****************** Select Test: *********************"\
                            "\n\r*****************************************************"\
                            "\n\r*   Press any key to receive its echo(ASCII+1)!     *"\
                            "\n\r*****************************************************"\
                            "\n\r\n\r>\0");
}

#endif	//RAM_Functions_Enabled



/**********************************************************************************/
/*      USART, Rx Complete interrupt service routine.                             */
/**********************************************************************************/

/**********************************************************************************/
/*  Function name: USCIAB0RX_ISR                                         	  */
/*  	Parameters                                                                */
/*          Input   :  No			     		                  */
/*          Output  :  No                                      	            	  */
/*	Action: UART receive interrupt.                          		  */
/**********************************************************************************/
#pragma vector=USCIAB0RX_VECTOR
__interrupt void USCIAB0RX_ISR(void){
  
  USART_Received_Data = USART_RxData_Reg;   // Read received data
  fUSART_Receive_Set;                       // Set flag
  //Return Echo:
  USART_Received_Data++;
  USART_Send_Data(USART_Received_Data);
  
  // Exit Interrupt
  IFG2 &= ~0x01;  // Clear USCI_A0 receive interrupt flag.
  
}





